/*
// Copyright (c) 2018 Intel Corporation
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//      http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
*/

///////////////////////////////////////////////////////////////////////////////////////////////////
#pragma once
#include "api/CPP/embed.hpp"
#include "primitive_inst.h"

namespace cldnn
{
	template <>
	struct typed_program_node<embed> : public typed_program_node_base<embed>
	{
		using parent = typed_program_node_base<embed>;

	public:
		using parent::parent;

		program_node& input() const { return get_dependency(0); }
		program_node& weights() const { return get_dependency(1); }
		program_node& bias() const { return get_dependency(2); }
		bool bias_term() const { return !get_primitive()->bias.empty(); }
	};

	using embed_node = typed_program_node<embed>;

	template <>
	class typed_primitive_inst<embed> : public typed_primitive_inst_base<embed>
	{
		using parent = typed_primitive_inst_base<embed>;

	public:
		static layout calc_output_layout(embed_node const& node);
		static std::string to_string(embed_node const& node);

	public:
		typed_primitive_inst(network_impl& network, embed_node const& node);
		memory_impl& weights_memory() const { return dep_memory(1); }
		memory_impl& bias_memory() const { return dep_memory(2); }
		bool bias_term() const { return !argument.bias.empty(); }
	};

	using embed_inst = typed_primitive_inst<embed>;

}
