/*
// Copyright (c) 2016 Intel Corporation
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//      http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
*/

///////////////////////////////////////////////////////////////////////////////////////////////////
#include "input_layout_inst.h"
#include "primitive_type_base.h"
#include "memory_impl.h"
#include "error_handler.h"
#include "json_object.h"

namespace cldnn
{
primitive_type_id input_layout_type_id()
{
    static primitive_type_base<input_layout> instance;
    return &instance;
}

input_layout_inst::typed_primitive_inst(network_impl& network, input_layout_node const& node)
    : parent(network, node)
{
    _has_valid_input = false; //by default input for 'input_layout' is invalid as long as user doesn't call set_data
}

void input_layout_inst::set_data(memory_impl& mem)
{

    CLDNN_ERROR_LAYOUT_MISMATCH("input layout", "memory layout", mem.get_layout(), "output memory layout", node.get_output_layout(), "");

    if (mem.is_allocated_by(get_network().get_engine()))
    {
        _output = &mem;
    }
    else
    {
        mem_lock<char> src(&mem);
        mem_lock<char> dst(_output);
        std::copy(src.begin(), src.end(), dst.begin());
    }

    _has_valid_input = true;
    _output_changed = true;
}

std::string input_layout_inst::to_string(input_layout_node const& node)
{
    auto node_info = node.desc_to_json();

    std::stringstream primitive_description;

    node_info->dump(primitive_description);

    return primitive_description.str();
}

}
